﻿'# Title    : SDX9 TerrainMesh
'# Author   : Sascha Bajus
'# Created  : 04/2010
'# Copyright © 2010 Sascha Bajus



#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region

Imports SlimDX
Imports SlimDX.Direct3D9
Imports SlimDX.DirectInput
Imports sdxBaseEngine


Public Class Project : Inherits SDX9BaseEngine

    Private _terrain As New shaderTerrainMesh
    Private _texset As New shaderTerrainMesh.TextureSet
    Private _light As New shaderLight.LightSet


    Public Sub New()

        AddHandler dxRenderScene, AddressOf render                      '#Render the Scene
        AddHandler dxKeyboardState, AddressOf keyboard                  '#Get Keyboard Inputs
        AddHandler dxMouseState, AddressOf mouse                        '#Get Mouse Inputs
        AddHandler dxLoadObjectsSettings, AddressOf LoadObjectsSettings '#Load all Settings
        AddHandler dxLoadObjectsTextures, AddressOf LoadObjectsTextures '#Load Textures (Autoreload if Device is Lost)
        AddHandler dxEngineClosed, AddressOf ExitAplication             '#Exit and Dispose all

        '---------------------------------------------

        'dxEngine.KeyboardSetCooperativeLevel = CooperativeLevel.Exclusive Or CooperativeLevel.Foreground
        dxEngine.MouseSetCooperativeLevel = CooperativeLevel.Exclusive Or CooperativeLevel.Foreground

        If Me.dxEngine("SlimDX - SDX9 TerrainMesh") = False Then CloseBaseEngine()
    End Sub

#Region "## Loading ObjectsSettings ##"
    Private Sub LoadObjectsSettings()

        '#loading resource to engine bytebuffer
        dxEngine.ResourceLoader.LoadResourceFile = "\data\resx.data"

        '#settings Camera rotate
        With dxCamera
            Dim _set As New engineCamera.cameraSetMouseRotate
            _set.Name = ""
            _set.Radius = 22
            _set.Speed = New Vector2(10, 40)
            _set.Height = 6
            _set.useSmoothRotations = True
            .useCameraSet(_set)
            .currentPlaneNear = 0.1
        End With

        '#settings light
        With _light
            .Diffuse = New RGBAcolor(1.0)                 '#diffuse color
            .DiffuseIntensity = 1.2                       '#diffuse intensity
            .Ambient = New RGBAcolor(1.0)                 '#ambient color
            .AmbientIntensity = 0.2                       '#ambient intensity
            .LightDirection = New Vector3(4.5, -1.9, 4.5)  '#light direction
        End With

        '#terrain settings
        With _terrain
            .MeshFromEngineResource("model\terrain.x")
            .Brightness = 1.4
            .TextureRepeatOffset = New Vector4(6, 6, 6, 6)
            .TextureTechnique = shaderTerrainMesh.Technique.Texture
        End With

    End Sub
#End Region
#Region "## Loading ObjectsTextures##"
    Private Sub LoadObjectsTextures()
        _texset.ColorMap0 = dxTexture.FromEngineResource("texture\T_Sand_amb.dds", Color.Black)
        _texset.ColorMap1 = dxTexture.FromEngineResource("texture\T_Grass_amb.dds", Color.Black)
        _texset.ColorMap2 = dxTexture.FromEngineResource("texture\T_Rock_amb.dds", Color.Black)

        _texset.NormalMap0 = dxTexture.FromEngineResource("texture\T_Sand_NORM.dds", Color.Black)
        _texset.NormalMap1 = dxTexture.FromEngineResource("texture\T_Grass_NORM.dds", Color.Black)
        _texset.NormalMap2 = dxTexture.FromEngineResource("texture\T_Rock_NORM.dds", Color.Black)
    End Sub
#End Region

#Region "## render ##"
    Private Sub render()
        dxTransformWorld = Matrix.Identity
        _terrain.Textures = _texset
        _terrain.DrawFX(_light)
        dx2DText.DrawFPS("Press Key 1" & vbCrLf & "Press Key 2", ":  Textured" & vbCrLf & ":  NormalMapping")
    End Sub


#End Region
#Region "## Inputdevices ##"
    Private Sub keyboard()

        If dxEngine.KeyPressed(Key.Escape) Then CloseBaseEngine()
        If dxEngine.KeyPressed(SlimDX.DirectInput.Key.F12) Then dxEngine.SaveScreenshot("SDX9 TerrainMesh")

        If dxEngine.KeyPressed(SlimDX.DirectInput.Key.D1) Then _terrain.TextureTechnique = shaderTerrainMesh.Technique.Texture
        If dxEngine.KeyPressed(SlimDX.DirectInput.Key.D2) Then _terrain.TextureTechnique = shaderTerrainMesh.Technique.TextureNormal

    End Sub

    Private Sub mouse()
        dxCamera.useMouseRotation = dxEngine.MouseMovement
    End Sub
#End Region

#Region "Exit App"
    Private Sub ExitAplication()
        If dxEngine.EngineError Then
            Me.Hide()
            MsgBox(dxEngine.EngineErrorDescription, MsgBoxStyle.Critical)
        End If
        End
    End Sub
#End Region

    Private Sub InitializeComponent()
        Dim resources As System.ComponentModel.ComponentResourceManager = New System.ComponentModel.ComponentResourceManager(GetType(Project))
        Me.SuspendLayout()
        '
        'Project
        '
        Me.ClientSize = New System.Drawing.Size(284, 262)
        Me.Icon = CType(resources.GetObject("$this.Icon"), System.Drawing.Icon)
        Me.Name = "Project"
        Me.ResumeLayout(False)

    End Sub
End Class
